@@ -5,8 +5,9 @@ import subprocess
 from textwrap import dedent
 
 import py
-from tox._config import IniReader, CommandParser
-from tox._config import parseconfig
+from tox._config import *
+from tox._config import _split_env
+
 
 class TestVenvConfig:
     def test_config_parsing_minimal(self, tmpdir, newconfig):
@@ -44,6 +45,22 @@ class TestVenvConfig:
         assert dep2.indexserver.name == "xyz"
         assert dep2.indexserver.url == "xyz_repo"
 
+    def test_envdir_set_manually(self, tmpdir, newconfig):
+        config = newconfig([], """
+            [testenv:devenv]
+            envdir = devenv
+        """)
+        envconfig = config.envconfigs['devenv']
+        assert envconfig.envdir == tmpdir.join('devenv')
+
+    def test_envdir_set_manually_with_substitutions(self, tmpdir, newconfig):
+        config = newconfig([], """
+            [testenv:devenv]
+            envdir = {toxworkdir}/foobar
+        """)
+        envconfig = config.envconfigs['devenv']
+        assert envconfig.envdir == config.toxworkdir.join('foobar')
+
 class TestConfigPackage:
     def test_defaults(self, tmpdir, newconfig):
         config = newconfig([], "")
@@ -303,6 +320,16 @@ class TestIniParser:
         assert reader.getargvlist('section', 'key')[0] == expected
 
 
+    def test_getargv(self, newconfig):
+        config = newconfig("""
+            [section]
+            key=some command "with quoting"
+        """)
+        reader = IniReader(config._cfg)
+        expected = ['some', 'command', 'with quoting']
+        assert reader.getargv('section', 'key') == expected
+
+
     def test_getpath(self, tmpdir, newconfig):
         config = newconfig("""
             [section]
@@ -354,6 +381,7 @@ class TestConfigTestEnv:
         assert envconfig.changedir == config.setupdir
         assert envconfig.distribute == False
         assert envconfig.sitepackages == False
+        assert envconfig.develop == False
         assert envconfig.envlogdir == envconfig.envdir.join("log")
         assert envconfig.setenv is None
 
@@ -445,6 +473,76 @@ class TestConfigTestEnv:
         assert envconfig.changedir.basename == "abc"
         assert envconfig.changedir == config.setupdir.join("abc")
 
+    def test_install_command_defaults_py25(self, newconfig, monkeypatch):
+        from tox.interpreters import Interpreters
+        def get_info(self, name):
+            if "x25" in name:
+                class I:
+                    runnable = True
+                    executable = "python2.5"
+                    version_info = (2,5)
+            else:
+                class I:
+                    runnable = False
+                    executable = "python"
+            return I
+        monkeypatch.setattr(Interpreters, "get_info", get_info)
+        config = newconfig("""
+            [testenv:x25]
+            basepython = x25
+            [testenv:py25-x]
+            basepython = x25
+            [testenv:py26]
+            basepython = "python"
+        """)
+        for name in ("x25", "py25-x"):
+            env = config.envconfigs[name]
+            assert env.install_command == \
+               "pip install --insecure {opts} {packages}".split()
+        env = config.envconfigs["py26"]
+        assert env.install_command == \
+               "pip install --pre {opts} {packages}".split()
+
+    def test_install_command_setting(self, newconfig):
+        config = newconfig("""
+            [testenv]
+            install_command=some_install {packages}
+        """)
+        envconfig = config.envconfigs['python']
+        assert envconfig.install_command == [
+            'some_install', '{packages}']
+
+    def test_install_command_must_contain_packages(self, newconfig):
+        py.test.raises(tox.exception.ConfigError, newconfig, """
+            [testenv]
+            install_command=pip install
+        """)
+
+    def test_downloadcache(self, newconfig, monkeypatch):
+        monkeypatch.delenv("PIP_DOWNLOAD_CACHE", raising=False)
+        config = newconfig("""
+            [testenv]
+            downloadcache=thecache
+        """)
+        envconfig = config.envconfigs['python']
+        assert envconfig.downloadcache.basename == 'thecache'
+
+    def test_downloadcache_env_override(self, newconfig, monkeypatch):
+        monkeypatch.setenv("PIP_DOWNLOAD_CACHE", 'fromenv')
+        config = newconfig("""
+            [testenv]
+            downloadcache=somepath
+        """)
+        envconfig = config.envconfigs['python']
+        assert envconfig.downloadcache.basename == "fromenv"
+
+    def test_downloadcache_only_if_in_config(self, newconfig, tmpdir,
+                                             monkeypatch):
+        monkeypatch.setenv("PIP_DOWNLOAD_CACHE", tmpdir)
+        config = newconfig('')
+        envconfig = config.envconfigs['python']
+        assert not envconfig.downloadcache
+
     def test_simple(tmpdir, newconfig):
         config = newconfig("""
             [testenv:py24]
@@ -632,36 +730,6 @@ class TestConfigTestEnv:
         assert conf.changedir.basename == 'testing'
         assert conf.changedir.dirpath().realpath() == tmpdir.realpath()
 
-    @pytest.mark.xfailif("sys.platform == 'win32'")
-    def test_substitution_envsitepackagesdir(self, tmpdir, monkeypatch,
-                                             newconfig):
-        """
-         The envsitepackagesdir property is mostly doing system work,
-         so this test doesn't excercise it very well.
-
-         Usage of envsitepackagesdir on win32/jython will explicitly
-         throw an exception,
-        """
-        class MockPopen(object):
-            returncode = 0
-
-            def __init__(self, *args, **kwargs):
-                pass
-
-            def communicate(self, *args, **kwargs):
-                return 'onevalue', 'othervalue'
-
-        monkeypatch.setattr(subprocess, 'Popen', MockPopen)
-        env = 'py%s' % (''.join(sys.version.split('.')[0:2]))
-        config = newconfig("""
-            [testenv:%s]
-            commands = {envsitepackagesdir}
-        """ % (env))
-        conf = config.envconfigs[env]
-        argv = conf.commands
-        assert argv[0][0] == 'onevalue'
-
-
 class TestGlobalOptions:
     def test_notest(self, newconfig):
         config = newconfig([], "")
@@ -677,7 +745,8 @@ class TestGlobalOptions:
         config = newconfig(["-vv"], "")
         assert config.option.verbosity == 2
 
-    def test_substitution_jenkins_default(self, tmpdir, monkeypatch, newconfig):
+    def test_substitution_jenkins_default(self, tmpdir,
+                                          monkeypatch, newconfig):
         monkeypatch.setenv("HUDSON_URL", "xyz")
         config = newconfig("""
             [testenv:py24]
@@ -910,6 +979,34 @@ class TestCmdInvocation:
             "*ERROR*tox.ini*not*found*",
         ])
 
+
+class TestArgumentParser:
+
+    def test_dash_e_single_1(self):
+        parser = prepare_parse('testpkg')
+        args = parser.parse_args('-e py26'.split())
+        envlist = _split_env(args.env)
+        assert envlist == ['py26']
+
+    def test_dash_e_single_2(self):
+        parser = prepare_parse('testpkg')
+        args = parser.parse_args('-e py26,py33'.split())
+        envlist = _split_env(args.env)
+        assert envlist == ['py26', 'py33']
+
+    def test_dash_e_same(self):
+        parser = prepare_parse('testpkg')
+        args = parser.parse_args('-e py26,py26'.split())
+        envlist = _split_env(args.env)
+        assert envlist == ['py26', 'py26']
+
+    def test_dash_e_combine(self):
+        parser = prepare_parse('testpkg')
+        args = parser.parse_args('-e py26,py25,py33 -e py33,py27'.split())
+        envlist = _split_env(args.env)
+        assert envlist == ['py26', 'py25', 'py33', 'py33', 'py27']
+
+
 class TestCommandParser:
 
     def test_command_parser_for_word(self):
