@@ -2,8 +2,9 @@ import py
 import tox
 import pytest
 import os, sys
-from tox._venv import VirtualEnv, CreationConfig, getdigest
-from tox._venv import find_executable
+from tox._venv import *
+
+py25calls = int(sys.version_info[:2] == (2,5))
 
 #def test_global_virtualenv(capfd):
 #    v = VirtualEnv()
@@ -16,52 +17,6 @@ from tox._venv import find_executable
 def test_getdigest(tmpdir):
     assert getdigest(tmpdir) == "0"*32
 
-@pytest.mark.skipif("sys.platform != 'win32'")
-def test_locate_via_py(monkeypatch):
-    from tox._venv import locate_via_py
-    class PseudoPy:
-        def sysexec(self, *args):
-            assert args[0] == '-3.2'
-            assert args[1] == '-c'
-            # Return value needs to actually exist!
-            return sys.executable
-    @staticmethod
-    def ret_pseudopy(name):
-        assert name == 'py'
-        return PseudoPy()
-    # Monkeypatch py.path.local.sysfind to return PseudoPy
-    monkeypatch.setattr(py.path.local, 'sysfind', ret_pseudopy)
-    assert locate_via_py('3', '2') == sys.executable
-
-def test_find_executable():
-    p = find_executable(sys.executable)
-    assert p == py.path.local(sys.executable)
-    for ver in [""] + "2.4 2.5 2.6 2.7 3.0 3.1 3.2 3.3".split():
-        name = "python%s" % ver
-        if sys.platform == "win32":
-            pydir = "python%s" % ver.replace(".", "")
-            x = py.path.local("c:\%s" % pydir)
-            print (x)
-            if not x.check():
-                continue
-        else:
-            if not py.path.local.sysfind(name):
-                continue
-        p = find_executable(name)
-        assert p
-        popen = py.std.subprocess.Popen([str(p), '-V'],
-                stderr=py.std.subprocess.PIPE)
-        stdout, stderr = popen.communicate()
-        assert ver in py.builtin._totext(stderr, "ascii")
-
-def test_find_executable_extra(monkeypatch):
-    @staticmethod
-    def sysfind(x):
-        return "hello"
-    monkeypatch.setattr(py.path.local, "sysfind", sysfind)
-    t = find_executable("qweqwe")
-    assert t == "hello"
-
 def test_getsupportedinterpreter(monkeypatch, newconfig, mocksession):
     config = newconfig([], """
         [testenv:python]
@@ -80,6 +35,7 @@ def test_getsupportedinterpreter(monkeypatch, newconfig, mocksession):
     py.test.raises(tox.exception.InterpreterNotFound,
                    venv.getsupportedinterpreter)
 
+
 def test_create(monkeypatch, mocksession, newconfig):
     config = newconfig([], """
         [testenv:py123]
@@ -92,14 +48,14 @@ def test_create(monkeypatch, mocksession, newconfig):
     l = mocksession._pcalls
     assert len(l) >= 1
     args = l[0].args
-    assert str(args[1]).endswith("virtualenv.py")
+    assert "virtualenv" in str(args[1])
     if sys.platform != "win32":
         # realpath is needed for stuff like the debian symlinks
         assert py.path.local(sys.executable).realpath() == args[0]
         #assert Envconfig.toxworkdir in args
         assert venv.getcommandpath("easy_install", cwd=py.path.local())
     interp = venv._getliveconfig().python
-    assert interp == venv.getconfigexecutable()
+    assert interp == venv.envconfig._basepython_info.executable
     assert venv.path_config.check(exists=False)
 
 @pytest.mark.skipif("sys.platform == 'win32'")
@@ -167,15 +123,15 @@ def test_install_deps_wildcard(newmocksession):
     venv = mocksession.getenv("py123")
     venv.create()
     l = mocksession._pcalls
-    assert len(l) == 1
+    assert len(l) == 1 + py25calls
     distshare = venv.session.config.distshare
     distshare.ensure("dep1-1.0.zip")
     distshare.ensure("dep1-1.1.zip")
 
     venv.install_deps()
-    assert len(l) == 2
-    args = l[1].args
-    assert l[1].cwd == venv.envconfig.envlogdir
+    assert len(l) == 2 + py25calls
+    args = l[-1].args
+    assert l[-1].cwd == venv.envconfig.envlogdir
     assert "pip" in str(args[0])
     assert args[1] == "install"
     #arg = "--download-cache=" + str(venv.envconfig.downloadcache)
@@ -200,18 +156,14 @@ def test_install_downloadcache(newmocksession, monkeypatch, tmpdir, envdc):
     venv = mocksession.getenv("py123")
     venv.create()
     l = mocksession._pcalls
-    assert len(l) == 1
+    assert len(l) == 1 + py25calls
 
     venv.install_deps()
-    assert len(l) == 2
-    args = l[1].args
-    assert l[1].cwd == venv.envconfig.envlogdir
+    assert len(l) == 2 + py25calls
+    args = l[-1].args
+    assert l[-1].cwd == venv.envconfig.envlogdir
     assert "pip" in str(args[0])
     assert args[1] == "install"
-    if envdc:
-        assert venv.envconfig.downloadcache == tmpdir
-    else:
-        assert not venv.envconfig.downloadcache
     assert "dep1" in args
     assert "dep2" in args
     deps = list(filter(None, [x[1] for x in venv._getliveconfig().deps]))
@@ -232,14 +184,14 @@ def test_install_deps_indexserver(newmocksession):
     venv = mocksession.getenv('py123')
     venv.create()
     l = mocksession._pcalls
-    assert len(l) == 1
+    assert len(l) == 1 + py25calls
     l[:] = []
 
     venv.install_deps()
     # two different index servers, two calls
     assert len(l) == 3
     args = " ".join(l[0].args)
-    assert "-i" not in args
+    assert "-i " not in args
     assert "dep1" in args
 
     args = " ".join(l[1].args)
@@ -264,14 +216,15 @@ def test_installpkg_indexserver(newmocksession, tmpdir):
     args = " ".join(l[0].args)
     assert "-i ABC" in args
 
-def test_install_recreate(newmocksession):
+def test_install_recreate(newmocksession, tmpdir):
+    pkg = tmpdir.ensure("package.tar.gz")
     mocksession = newmocksession(['--recreate'], """
         [testenv]
         deps=xyz
     """)
     venv = mocksession.getenv('python')
     venv.update()
-    mocksession.installpkg(venv, "xz")
+    mocksession.installpkg(venv, pkg)
     mocksession.report.expect("verbosity0", "*create*")
     venv.update()
     mocksession.report.expect("verbosity0", "*recreate*")
@@ -426,14 +379,15 @@ class TestCreationConfig:
         assert path == xyz2
         assert md5 == path.computehash()
 
-    def test_python_recreation(self, newconfig, mocksession):
+    def test_python_recreation(self, tmpdir, newconfig, mocksession):
+        pkg = tmpdir.ensure("package.tar.gz")
         config = newconfig([], "")
         envconfig = config.envconfigs['python']
         venv = VirtualEnv(envconfig, session=mocksession)
         cconfig = venv._getliveconfig()
         venv.update()
         assert not venv.path_config.check()
-        mocksession.installpkg(venv, "sdist.zip")
+        mocksession.installpkg(venv, pkg)
         assert venv.path_config.check()
         assert mocksession._pcalls
         args1 = map(str, mocksession._pcalls[0].args)
@@ -473,6 +427,18 @@ class TestCreationConfig:
         venv.update()
         mocksession.report.expect("verbosity0", "*recreate*")
 
+    def test_develop_recreation(self, newconfig, mocksession):
+        config = newconfig([], "")
+        envconfig = config.envconfigs['python']
+        venv = VirtualEnv(envconfig, session=mocksession)
+        venv.update()
+        cconfig = venv._getliveconfig()
+        cconfig.develop = True
+        cconfig.writeconfig(venv.path_config)
+        mocksession._clearmocks()
+        venv.update()
+        mocksession.report.expect("verbosity0", "*recreate*")
+
 class TestVenvTest:
 
     def test_patchPATH(self, newmocksession, monkeypatch):
@@ -498,16 +464,16 @@ class TestVenvTest:
         monkeypatch.setattr(venv, '_pcall', lambda *args, **kwargs: 0/0)
         py.test.raises(ZeroDivisionError, "venv._install(list('123'))")
         py.test.raises(ZeroDivisionError, "venv.test()")
-        py.test.raises(ZeroDivisionError, "venv.easy_install(['qwe'])")
-        py.test.raises(ZeroDivisionError, "venv.pip_install(['qwe'])")
+        py.test.raises(ZeroDivisionError, "venv.run_install_command(['qwe'])")
         py.test.raises(ZeroDivisionError, "venv._pcall([1,2,3])")
         monkeypatch.setenv("PIP_RESPECT_VIRTUALENV", "1")
         monkeypatch.setenv("PIP_REQUIRE_VIRTUALENV", "1")
-        py.test.raises(ZeroDivisionError, "venv.pip_install(['qwe'])")
+        py.test.raises(ZeroDivisionError, "venv.run_install_command(['qwe'])")
         assert 'PIP_RESPECT_VIRTUALENV' not in os.environ
         assert 'PIP_REQUIRE_VIRTUALENV' not in os.environ
 
-def test_setenv_added_to_pcall(mocksession, newconfig):
+def test_setenv_added_to_pcall(tmpdir, mocksession, newconfig):
+    pkg = tmpdir.ensure("package.tar.gz")
     config = newconfig([], """
         [testenv:python]
         commands=python -V
@@ -518,7 +484,7 @@ def test_setenv_added_to_pcall(mocksession, newconfig):
 
     venv = VirtualEnv(config.envconfigs['python'], session=mocksession)
     # import pdb; pdb.set_trace()
-    mocksession.installpkg(venv, "xyz")
+    mocksession.installpkg(venv, pkg)
     venv.test()
 
     l = mocksession._pcalls
@@ -533,33 +499,35 @@ def test_setenv_added_to_pcall(mocksession, newconfig):
     for e in os.environ:
         assert e in env
 
-def test_installpkg_no_upgrade(newmocksession):
+def test_installpkg_no_upgrade(tmpdir, newmocksession):
+    pkg = tmpdir.ensure("package.tar.gz")
     mocksession = newmocksession([], "")
     venv = mocksession.getenv('python')
     venv.just_created = True
     venv.envconfig.envdir.ensure(dir=1)
-    mocksession.installpkg(venv, "whatever")
+    mocksession.installpkg(venv, pkg)
     l = mocksession._pcalls
     assert len(l) == 1
     assert '-U' not in l[0].args
 
-def test_installpkg_upgrade(newmocksession):
+def test_installpkg_upgrade(newmocksession, tmpdir):
+    pkg = tmpdir.ensure("package.tar.gz")
     mocksession = newmocksession([], "")
     venv = mocksession.getenv('python')
     assert not hasattr(venv, 'just_created')
-    mocksession.installpkg(venv, "whatever")
+    mocksession.installpkg(venv, pkg)
     l = mocksession._pcalls
     assert len(l) == 1
     assert '-U' in l[0].args
     assert '--no-deps' in l[0].args
 
-def test_pip_install(newmocksession):
+def test_run_install_command(newmocksession):
     mocksession = newmocksession([], "")
     venv = mocksession.getenv('python')
     venv.just_created = True
     venv.envconfig.envdir.ensure(dir=1)
     action = mocksession.newaction(venv, "hello")
-    venv.pip_install(args=["whatever"], action=action)
+    venv.run_install_command(args=["whatever"], action=action)
     l = mocksession._pcalls
     assert len(l) == 1
     assert 'pip' in l[0].args[0]
@@ -569,6 +537,21 @@ def test_pip_install(newmocksession):
     assert 'PYTHONIOENCODING' in env
     assert env['PYTHONIOENCODING'] == 'utf_8'
 
+def test_run_custom_install_command(newmocksession):
+    mocksession = newmocksession([], """
+        [testenv]
+        install_command=easy_install {opts} {packages}
+    """)
+    venv = mocksession.getenv('python')
+    venv.just_created = True
+    venv.envconfig.envdir.ensure(dir=1)
+    action = mocksession.newaction(venv, "hello")
+    venv.run_install_command(args=["whatever"], action=action)
+    l = mocksession._pcalls
+    assert len(l) == 1
+    assert 'easy_install' in l[0].args[0]
+    assert l[0].args[1:] == ['whatever']
+
 def test_command_relative_issue26(newmocksession, tmpdir, monkeypatch):
     mocksession = newmocksession([], """
         [testenv]
@@ -584,3 +567,28 @@ def test_command_relative_issue26(newmocksession, tmpdir, monkeypatch):
     monkeypatch.setenv("PATH", str(tmpdir))
     x4 = venv.getcommandpath("x", cwd=tmpdir)
     mocksession.report.expect("warning", "*test command found but not*")
+
+def test_hack_home_env(tmpdir):
+    from tox._venv import hack_home_env
+    env = hack_home_env(tmpdir, "http://index")
+    assert env["HOME"] == str(tmpdir)
+    assert env["PIP_INDEX_URL"] == "http://index"
+    assert "index_url = http://index" in \
+           tmpdir.join(".pydistutils.cfg").read()
+    tmpdir.remove()
+    env = hack_home_env(tmpdir, None)
+    assert env["HOME"] == str(tmpdir)
+    assert not tmpdir.join(".pydistutils.cfg").check()
+    assert "PIP_INDEX_URL" not in env
+
+def test_hack_home_env_passthrough(tmpdir, monkeypatch):
+    from tox._venv import hack_home_env
+    env = hack_home_env(tmpdir, "http://index")
+    monkeypatch.setattr(os, "environ", env)
+
+    tmpdir = tmpdir.mkdir("tmpdir2")
+    env2 = hack_home_env(tmpdir)
+    assert env2["HOME"] == str(tmpdir)
+    assert env2["PIP_INDEX_URL"] == "http://index"
+    assert "index_url = http://index" in \
+           tmpdir.join(".pydistutils.cfg").read()
