@@ -3,6 +3,10 @@ import py
 import pytest
 import sys
 from tox._pytestplugin import ReportExpectMock
+try:
+    import json
+except ImportError:
+    import simplejson as json
 
 pytest_plugins = "pytester"
 
@@ -252,6 +256,71 @@ def test_unknown_environment(cmd, initproj):
         "*ERROR*unknown*environment*qpwoei*",
     ])
 
+def test_skip_sdist(cmd, initproj):
+    initproj("pkg123-0.7", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+        'setup.py': """
+            syntax error
+        """
+        ,
+        'tox.ini': '''
+            [tox]
+            skipsdist=True
+            [testenv]
+            commands=python -c "print('done')"
+        '''
+    })
+    result = cmd.run("tox", )
+    assert result.ret == 0
+
+def test_minimal_setup_py_empty(cmd, initproj):
+    initproj("pkg123-0.7", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+        'setup.py': """
+        """
+        ,
+        'tox.ini': ''
+
+    })
+    result = cmd.run("tox", )
+    assert result.ret == 1
+    result.stdout.fnmatch_lines([
+        "*ERROR*empty*",
+    ])
+
+def test_minimal_setup_py_comment_only(cmd, initproj):
+    initproj("pkg123-0.7", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+        'setup.py': """\n# some comment
+
+        """
+        ,
+        'tox.ini': ''
+
+    })
+    result = cmd.run("tox", )
+    assert result.ret == 1
+    result.stdout.fnmatch_lines([
+        "*ERROR*empty*",
+    ])
+
+def test_minimal_setup_py_non_functional(cmd, initproj):
+    initproj("pkg123-0.7", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+        'setup.py': """
+        import sys
+
+        """
+        ,
+        'tox.ini': ''
+
+    })
+    result = cmd.run("tox", )
+    assert result.ret == 1
+    result.stdout.fnmatch_lines([
+        "*ERROR*check setup.py*",
+    ])
+
 def test_sdist_fails(cmd, initproj):
     initproj("pkg123-0.7", filedefs={
         'tests': {'test_hello.py': "def test_hello(): pass"},
@@ -291,7 +360,104 @@ def test_package_install_fails(cmd, initproj):
         "*InvocationError*",
     ])
 
-def test_test_simple(cmd, initproj):
+
+
+class TestToxRun:
+    @pytest.fixture
+    def example123(self, initproj):
+        initproj("example123-0.5", filedefs={
+            'tests': {'test_hello.py': """
+                def test_hello(pytestconfig):
+                    pass
+                """,
+            },
+            'tox.ini': '''
+                [testenv]
+                changedir=tests
+                commands= py.test --basetemp={envtmpdir} \
+                                  --junitxml=junit-{envname}.xml
+                deps=pytest
+            '''
+        })
+
+    def test_toxuone_env(self, cmd, example123):
+        result = cmd.run("tox")
+        assert not result.ret
+        result.stdout.fnmatch_lines([
+            "*junit-python.xml*",
+            "*1 passed*",
+        ])
+        result = cmd.run("tox", "-epython", )
+        assert not result.ret
+        result.stdout.fnmatch_lines([
+            "*1 passed*",
+            "*summary*",
+            "*python: commands succeeded"
+        ])
+
+    def test_different_config_cwd(self, cmd, example123, monkeypatch):
+        # see that things work with a different CWD
+        monkeypatch.chdir(cmd.tmpdir)
+        result = cmd.run("tox", "-c", "example123/tox.ini")
+        assert not result.ret
+        result.stdout.fnmatch_lines([
+            "*1 passed*",
+            "*summary*",
+            "*python: commands succeeded"
+        ])
+
+    def test_json(self, cmd, example123):
+        # see that tests can also fail and retcode is correct
+        testfile = py.path.local("tests").join("test_hello.py")
+        assert testfile.check()
+        testfile.write("def test_fail(): assert 0")
+        jsonpath = cmd.tmpdir.join("res.json")
+        result = cmd.run("tox", "--result-json", jsonpath)
+        assert result.ret == 1
+        data = json.load(jsonpath.open("r"))
+        verify_json_report_format(data)
+        result.stdout.fnmatch_lines([
+            "*1 failed*",
+            "*summary*",
+            "*python: *failed*",
+        ])
+
+
+def test_develop(initproj, cmd):
+    initproj("example123", filedefs={'tox.ini': """
+    """})
+    result = cmd.run("tox", "-vv", "--develop")
+    assert not result.ret
+    assert "sdist-make" not in result.stdout.str()
+
+def test_usedevelop(initproj, cmd):
+    initproj("example123", filedefs={'tox.ini': """
+            [testenv]
+            usedevelop=True
+    """})
+    result = cmd.run("tox", "-vv")
+    assert not result.ret
+    assert "sdist-make" not in result.stdout.str()
+
+def test_usedevelop_mixed(initproj, cmd):
+    initproj("example123", filedefs={'tox.ini': """
+            [testenv:devenv]
+            usedevelop=True
+            [testenv:nondev]
+            usedevelop=False
+    """})
+
+    # running only 'devenv' should not do sdist
+    result = cmd.run("tox", "-vv", "-e", "devenv")
+    assert not result.ret
+    assert "sdist-make" not in result.stdout.str()
+
+    # running all envs should do sdist
+    result = cmd.run("tox", "-vv")
+    assert not result.ret
+    assert "sdist-make" in result.stdout.str()
+
+def test_test_usedevelop(cmd, initproj):
     initproj("example123-0.5", filedefs={
         'tests': {'test_hello.py': """
             def test_hello(pytestconfig):
@@ -300,18 +466,20 @@ def test_test_simple(cmd, initproj):
         },
         'tox.ini': '''
             [testenv]
+            usedevelop=True
             changedir=tests
             commands=
                 py.test --basetemp={envtmpdir} --junitxml=junit-{envname}.xml []
             deps=pytest
         '''
     })
-    result = cmd.run("tox")
+    result = cmd.run("tox", "-v")
     assert not result.ret
     result.stdout.fnmatch_lines([
         "*junit-python.xml*",
         "*1 passed*",
     ])
+    assert "sdist-make" not in result.stdout.str()
     result = cmd.run("tox", "-epython", )
     assert not result.ret
     result.stdout.fnmatch_lines([
@@ -347,10 +515,10 @@ def test_test_piphelp(initproj, cmd):
         # content of: tox.ini
         [testenv]
         commands=pip -h
-        [testenv:py25]
-        basepython=python
         [testenv:py26]
         basepython=python
+        [testenv:py27]
+        basepython=python
     """})
     result = cmd.run("tox")
     assert not result.ret
@@ -358,22 +526,22 @@ def test_test_piphelp(initproj, cmd):
 def test_notest(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
         # content of: tox.ini
-        [testenv:py25]
+        [testenv:py26]
         basepython=python
     """})
     result = cmd.run("tox", "-v", "--notest")
     assert not result.ret
     result.stdout.fnmatch_lines([
         "*summary*",
-        "*py25*skipped tests*",
+        "*py26*skipped tests*",
     ])
-    result = cmd.run("tox", "-v", "--notest", "-epy25")
+    result = cmd.run("tox", "-v", "--notest", "-epy26")
     assert not result.ret
     result.stdout.fnmatch_lines([
-        "*py25*reusing*",
+        "*py26*reusing*",
     ])
 
-def test_env_PYTHONDONTWRITEBYTECODE(initproj, cmd, monkeypatch):
+def test_PYC(initproj, cmd, monkeypatch):
     initproj("example123", filedefs={'tox.ini': ''})
     monkeypatch.setenv("PYTHONDOWNWRITEBYTECODE", 1)
     result = cmd.run("tox", "-v", "--notest")
@@ -382,6 +550,15 @@ def test_env_PYTHONDONTWRITEBYTECODE(initproj, cmd, monkeypatch):
         "*create*",
     ])
 
+def test_env_VIRTUALENV_PYTHON(initproj, cmd, monkeypatch):
+    initproj("example123", filedefs={'tox.ini': ''})
+    monkeypatch.setenv("VIRTUALENV_PYTHON", '/FOO')
+    result = cmd.run("tox", "-v", "--notest")
+    assert not result.ret, result.stdout.lines
+    result.stdout.fnmatch_lines([
+        "*create*",
+    ])
+
 def test_sdistonly(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
     """})
@@ -448,16 +625,33 @@ def test_installpkg(tmpdir, newconfig):
     sdist_path = session.sdist()
     assert sdist_path == p
 
-@pytest.mark.xfail("sys.platform == 'win32'", reason="test needs better impl")
+#@pytest.mark.xfail("sys.platform == 'win32'", reason="test needs better impl")
 def test_envsitepackagesdir(cmd, initproj):
     initproj("pkg512-0.0.5", filedefs={
         'tox.ini': """
         [testenv]
         commands=
-            echo X:{envsitepackagesdir}
+            python -c "print('X:{envsitepackagesdir}')"
     """})
     result = cmd.run("tox")
     assert result.ret == 0
     result.stdout.fnmatch_lines("""
         X:*site-packages*
     """)
+
+def verify_json_report_format(data, testenvs=True):
+    assert data["reportversion"] == "1"
+    assert data["toxversion"] == tox.__version__
+    if testenvs:
+        for envname, envdata in data["testenvs"].items():
+            for commandtype in ("setup", "test"):
+                if commandtype not in envdata:
+                    continue
+                for command in envdata[commandtype]:
+                    assert command["output"]
+                    assert command["retcode"]
+            pyinfo = envdata["python"]
+            assert isinstance(pyinfo["version_info"], list)
+            assert pyinfo["version"]
+            assert pyinfo["executable"]
+
