@@ -14,6 +14,7 @@ import subprocess
 from tox._verlib import NormalizedVersion, IrrationalVersionError
 from tox._venv import VirtualEnv
 from tox._config import parseconfig
+from tox.result import ResultLog
 from subprocess import STDOUT
 
 def now():
@@ -41,6 +42,10 @@ class Action(object):
             self.venvname = self.venv.name
         else:
             self.venvname = "GLOB"
+        cat = {"runtests": "test", "getenv": "setup"}.get(msg)
+        if cat:
+            envlog = session.resultlog.get_envlog(self.venvname)
+            self.commandlog = envlog.get_commandlog(cat)
 
     def __enter__(self):
         self.report.logaction_start(self)
@@ -73,21 +78,24 @@ class Action(object):
         f.flush()
         return f
 
-    def popen(self, args, cwd=None, env=None, redirect=True):
+    def popen(self, args, cwd=None, env=None, redirect=True, returnout=False):
         logged_command = "%s$ %s" %(cwd, " ".join(map(str, args)))
         f = outpath = None
-        if redirect:
+        resultjson = self.session.config.option.resultjson
+        if resultjson or redirect:
             f = self._initlogpath(self.id)
             f.write("actionid=%s\nmsg=%s\ncmdargs=%r\nenv=%s\n" %(
                     self.id, self.msg, args, env))
             f.flush()
             self.popen_outpath = outpath = py.path.local(f.name)
+        elif returnout:
+            f = subprocess.PIPE
         if cwd is None:
             # XXX cwd = self.session.config.cwd
             cwd = py.path.local()
         popen = self._popen(args, cwd, env=env, stdout=f, stderr=STDOUT)
         popen.outpath = outpath
-        popen.args = args
+        popen.args = [str(x) for x in args]
         popen.cwd = cwd
         popen.action = self
         self._popenlist.append(popen)
@@ -107,11 +115,18 @@ class Action(object):
             if outpath:
                 self.report.error("invocation failed, logfile: %s" %
                                   outpath)
-                self.report.error(outpath.read())
+                out = outpath.read()
+                self.report.error(out)
+                if hasattr(self, "commandlog"):
+                    self.commandlog.add_command(popen.args, out, ret)
                 raise tox.exception.InvocationError(
                     "%s (see %s)" %(invoked, outpath), ret)
             else:
                 raise tox.exception.InvocationError("%r" %(invoked, ))
+        if not out and outpath:
+            out = outpath.read()
+        if hasattr(self, "commandlog"):
+            self.commandlog.add_command(popen.args, out, ret)
         return out
 
     def _rewriteargs(self, cwd, args):
@@ -231,6 +246,7 @@ class Session:
     def __init__(self, config, popen=subprocess.Popen, Report=Reporter):
         self.config = config
         self.popen = popen
+        self.resultlog = ResultLog()
         self.report = Report(self)
         self.make_emptydir(config.logdir)
         config.logdir.ensure(dir=1)
@@ -305,7 +321,27 @@ class Session:
             action.popen([sys.executable, setup, "sdist", "--formats=zip",
                           "--dist-dir", self.config.distdir, ],
                           cwd=self.config.setupdir)
-            return self.config.distdir.listdir()[0]
+            try:
+                return self.config.distdir.listdir()[0]
+            except py.error.ENOENT:
+                # check if empty or comment only
+                data = []
+                with open(str(setup)) as fp:
+                    for line in fp:
+                        if line and line[0] == '#':
+                            continue
+                        data.append(line)
+                if not ''.join(data).strip():
+                    self.report.error(
+                        'setup.py is empty'
+                    )
+                    raise SystemExit(1)
+                self.report.error(
+                    'No dist directory found. Please check setup.py, e.g with:\n'\
+                    '     python setup.py sdist'
+                    )
+                raise SystemExit(1)
+
 
     def make_emptydir(self, path):
         if path.check():
@@ -317,17 +353,39 @@ class Session:
         action = self.newaction(venv, "getenv", venv.envconfig.envdir)
         with action:
             venv.status = 0
+            envlog = self.resultlog.get_envlog(venv.name)
             try:
                 status = venv.update(action=action)
             except tox.exception.InvocationError:
                 status = sys.exc_info()[1]
             if status:
+                commandlog = envlog.get_commandlog("setup")
+                commandlog.add_command(["setup virtualenv"], str(status), 1)
                 venv.status = status
                 self.report.error(str(status))
                 return False
+            commandpath = venv.getcommandpath("python")
+            envlog.set_python_info(commandpath)
             return True
 
+    def finishvenv(self, venv):
+        action = self.newaction(venv, "finishvenv")
+        with action:
+            venv.finish()
+            return True
+
+    def developpkg(self, venv, setupdir):
+        action = self.newaction(venv, "developpkg", setupdir)
+        with action:
+            try:
+                venv.developpkg(setupdir, action)
+                return True
+            except tox.exception.InvocationError:
+                venv.status = sys.exc_info()[1]
+                return False
+
     def installpkg(self, venv, sdist_path):
+        self.resultlog.set_header(installpkg=sdist_path)
         action = self.newaction(venv, "installpkg", sdist_path)
         with action:
             try:
@@ -362,19 +420,28 @@ class Session:
         return sdist_path
 
     def subcommand_test(self):
-        sdist_path = self.sdist()
-        if not sdist_path:
-            return 2
+        if self.config.skipsdist:
+            self.report.info("skipping sdist step")
+            sdist_path = None
+        else:
+            sdist_path = self.sdist()
+            if not sdist_path:
+                return 2
         if self.config.option.sdistonly:
             return
         for venv in self.venvlist:
             if self.setupenv(venv):
-                self.installpkg(venv, sdist_path)
-                self.runtestenv(venv, sdist_path)
+                if venv.envconfig.develop:
+                    self.developpkg(venv, self.config.setupdir)
+                elif self.config.skipsdist:
+                    self.finishvenv(venv)
+                else:
+                    self.installpkg(venv, sdist_path)
+                self.runtestenv(venv)
         retcode = self._summary()
         return retcode
 
-    def runtestenv(self, venv, sdist_path, redirect=False):
+    def runtestenv(self, venv, redirect=False):
         if not self.config.option.notest:
             if venv.status:
                 return
@@ -398,6 +465,12 @@ class Session:
                                               status))
         if not retcode:
             self.report.good("  congratulations :)")
+
+        path = self.config.option.resultjson
+        if path:
+            path = py.path.local(path)
+            path.write(self.resultlog.dumps_json())
+            self.report.line("wrote json report at: %s" % path)
         return retcode
 
     def showconfig(self):
@@ -408,22 +481,28 @@ class Session:
         self.report.keyvalue("toxworkdir: ", self.config.toxworkdir)
         self.report.keyvalue("setupdir:   ", self.config.setupdir)
         self.report.keyvalue("distshare:  ", self.config.distshare)
+        self.report.keyvalue("skipsdist:  ", self.config.skipsdist)
         self.report.tw.line()
         for envconfig in self.config.envconfigs.values():
             self.report.line("[testenv:%s]" % envconfig.envname, bold=True)
             self.report.line("  basepython=%s" % envconfig.basepython)
+            self.report.line("  _basepython_info=%s" %
+                             envconfig._basepython_info)
             self.report.line("  envpython=%s" % envconfig.envpython)
             self.report.line("  envtmpdir=%s" % envconfig.envtmpdir)
             self.report.line("  envbindir=%s" % envconfig.envbindir)
             self.report.line("  envlogdir=%s" % envconfig.envlogdir)
             self.report.line("  changedir=%s" % envconfig.changedir)
             self.report.line("  args_are_path=%s" % envconfig.args_are_paths)
+            self.report.line("  install_command=%s" %
+                             envconfig.install_command)
             self.report.line("  commands=")
             for command in envconfig.commands:
                 self.report.line("    %s" % command)
             self.report.line("  deps=%s" % envconfig.deps)
             self.report.line("  envdir=    %s" % envconfig.envdir)
             self.report.line("  downloadcache=%s" % envconfig.downloadcache)
+            self.report.line("  usedevelop=%s" % envconfig.develop)
 
     def showenvs(self):
         for env in self.config.envlist:
@@ -431,8 +510,12 @@ class Session:
 
     def info_versions(self):
         versions = ['tox-%s' % tox.__version__]
-        version = py.process.cmdexec("virtualenv --version")
-        versions.append("virtualenv-%s" % version.strip())
+        try:
+            version = py.process.cmdexec("virtualenv --version")
+        except py.process.cmdexec.Error:
+            versions.append("virtualenv-1.9.1 (vendored)")
+        else:
+            versions.append("virtualenv-%s" % version.strip())
         self.report.keyvalue("tool-versions:", " ".join(versions))
 
 
