@@ -8,6 +8,8 @@ import string
 import subprocess
 import textwrap
 
+from tox.interpreters import Interpreters
+
 import py
 
 import tox
@@ -76,14 +78,14 @@ def prepare_parse(pkgname):
     parser.add_argument("-v", nargs=0, action=CountAction, default=0,
         dest="verbosity",
         help="increase verbosity of reporting output.")
-    parser.add_argument("--showconfig", action="store_true", dest="showconfig",
-        help="show configuration information. ")
+    parser.add_argument("--showconfig", action="store_true",
+        help="show configuration information for all environments. ")
     parser.add_argument("-l", "--listenvs", action="store_true",
         dest="listenvs", help="show list of test environments")
     parser.add_argument("-c", action="store", default="tox.ini",
         dest="configfile",
         help="use the specified config file name.")
-    parser.add_argument("-e", action="store", dest="env",
+    parser.add_argument("-e", action="append", dest="env",
         metavar="envlist",
         help="work against specified environments (ALL selects all).")
     parser.add_argument("--notest", action="store_true", dest="notest",
@@ -91,7 +93,12 @@ def prepare_parse(pkgname):
     parser.add_argument("--sdistonly", action="store_true", dest="sdistonly",
         help="only perform the sdist packaging activity.")
     parser.add_argument("--installpkg", action="store", default=None,
-        help="use specified package for installation into venv")
+        metavar="PATH",
+        help="use specified package for installation into venv, instead of "
+             "creating an sdist.")
+    parser.add_argument("--develop", action="store_true", dest="develop",
+        help="install package in the venv using 'setup.py develop' via "
+             "'pip -e .'")
     parser.add_argument('-i', action="append",
         dest="indexurl", metavar="URL",
         help="set indexserver url (if URL is of form name=url set the "
@@ -99,6 +106,12 @@ def prepare_parse(pkgname):
     parser.add_argument("-r", "--recreate", action="store_true",
         dest="recreate",
         help="force recreation of virtual environments")
+    parser.add_argument("--result-json", action="store",
+        dest="resultjson", metavar="PATH",
+        help="write a json file with detailed information about "
+             "all commands and results involved.  This will turn off "
+             "pass-through output from running test commands which is "
+             "instead captured into the json result file.")
     parser.add_argument("args", nargs="*",
         help="additional arguments available to command positional substition")
     return parser
@@ -107,6 +120,7 @@ class Config:
     def __init__(self):
         self.envconfigs = {}
         self.invocationcwd = py.path.local()
+        self.interpreters = Interpreters()
 
 class VenvConfig:
     def __init__(self, **kw):
@@ -130,32 +144,10 @@ class VenvConfig:
 
     # no @property to avoid early calling (see callable(subst[key]) checks)
     def envsitepackagesdir(self):
-        print_envsitepackagesdir = textwrap.dedent("""
-        import sys
-        from distutils.sysconfig import get_python_lib
-        sys.stdout.write(get_python_lib(prefix=sys.argv[1]))
-        """)
-
-        exe = self.getsupportedinterpreter()
-        # can't use check_output until py27
-        proc = subprocess.Popen(
-            [str(exe), '-c', print_envsitepackagesdir, str(self.envdir)],
-            stdout=subprocess.PIPE)
-        odata, edata = proc.communicate()
-        if proc.returncode:
-            raise tox.exception.UnsupportedInterpreter(
-                "Error getting site-packages from %s" % self.basepython)
-        return odata
-
-    def getconfigexecutable(self):
-        from tox._venv import find_executable
-
-        python = self.basepython
-        if not python:
-            python = sys.executable
-        x = find_executable(str(python))
-        if x:
-            x = x.realpath()
+        self.getsupportedinterpreter()  # for throwing exceptions
+        x = self.config.interpreters.get_sitepackagesdir(
+                info=self._basepython_info,
+                envdir=self.envdir)
         return x
 
     def getsupportedinterpreter(self):
@@ -163,10 +155,11 @@ class VenvConfig:
                 "jython" in self.basepython:
             raise tox.exception.UnsupportedInterpreter(
                 "Jython/Windows does not support installing scripts")
-        config_executable = self.getconfigexecutable()
-        if not config_executable:
+        info = self.config.interpreters.get_info(self.basepython)
+        if not info.executable:
             raise tox.exception.InterpreterNotFound(self.basepython)
-        return config_executable
+        return info.executable
+
 testenvprefix = "testenv:"
 
 class parseini:
@@ -190,6 +183,8 @@ class parseini:
             raise ValueError("invalid context")
 
         config.homedir = py.path.local._gethomedir()
+        if config.homedir is None:
+            config.homedir = config.toxinidir  # XXX good idea?
         reader.addsubstitions(toxinidir=config.toxinidir,
                               homedir=config.homedir)
         config.toxworkdir = reader.getpath(toxsection, "toxworkdir",
@@ -250,11 +245,18 @@ class parseini:
                     config.envconfigs[name] = \
                 self._makeenvconfig(name, "_xz_9", reader._subs, config)
 
+        all_develop = all(name in config.envconfigs
+                          and config.envconfigs[name].develop
+                          for name in config.envlist)
+
+        config.skipsdist = reader.getbool(toxsection, "skipsdist", all_develop)
+
     def _makeenvconfig(self, name, section, subs, config):
         vc = VenvConfig(envname=name)
         vc.config = config
         reader = IniReader(self._cfg, fallbacksections=["testenv"])
         reader.addsubstitions(**subs)
+        vc.develop = reader.getbool(section, "usedevelop", config.option.develop)
         vc.envdir = reader.getpath(section, "envdir", "{toxworkdir}/%s" % name)
         vc.args_are_paths = reader.getbool(section, "args_are_paths", True)
         if reader.getdefault(section, "python", None):
@@ -265,6 +267,7 @@ class parseini:
         else:
             bp = sys.executable
         vc.basepython = reader.getdefault(section, "basepython", bp)
+        vc._basepython_info = config.interpreters.get_info(vc.basepython)
         reader.addsubstitions(envdir=vc.envdir, envname=vc.envname,
                               envbindir=vc.envbindir, envpython=vc.envpython,
                               envsitepackagesdir=vc.envsitepackagesdir)
@@ -306,11 +309,30 @@ class parseini:
         vc.distribute = reader.getbool(section, "distribute", False)
         vc.sitepackages = reader.getbool(section, "sitepackages", False)
         vc.downloadcache = None
-        downloadcache = os.environ.get("PIP_DOWNLOAD_CACHE", None)
-        if not downloadcache:
-            downloadcache = reader.getdefault(section, "downloadcache")
+        downloadcache = reader.getdefault(section, "downloadcache")
         if downloadcache:
+            # env var, if present, takes precedence
+            downloadcache = os.environ.get("PIP_DOWNLOAD_CACHE", downloadcache)
             vc.downloadcache = py.path.local(downloadcache)
+
+        # on python 2.5 we can't use "--pre" and we typically
+        # need to use --insecure for pip commands because python2.5
+        # doesn't support SSL
+        pip_default_opts = ["{opts}", "{packages}"]
+        info = vc._basepython_info
+        if info.runnable and info.version_info < (2,6):
+            pip_default_opts.insert(0, "--insecure")
+        else:
+            pip_default_opts.insert(0, "--pre")
+        vc.install_command = reader.getargv(
+            section,
+            "install_command",
+            "pip install " + " ".join(pip_default_opts),
+            replace=False,
+            )
+        if '{packages}' not in vc.install_command:
+            raise tox.exception.ConfigError(
+             "'install_command' must contain '{packages}' substitution")
         return vc
 
     def _getenvlist(self, reader, toxsection):
@@ -322,13 +344,24 @@ class parseini:
                 if not envlist:
                     envlist = self.config.envconfigs.keys()
                 return envlist
-        if env == "ALL":
+        envlist = _split_env(env)
+        if "ALL" in envlist:
             envlist = list(self.config.envconfigs)
             envlist.sort()
-        else:
-            envlist = env.split(",")
         return envlist
 
+def _split_env(env):
+    """if handed a list, action="append" was used for -e """
+    envlist = []
+    if not isinstance(env, list):
+        env = [env]
+    for to_split in env:
+        for single_env in to_split.split(","):
+            # "remove True or", if not allowing multiple same runs, update tests
+            if True or single_env not in envlist:
+                envlist.append(single_env)
+    return envlist
+
 class DepConfig:
     def __init__(self, name, indexserver=None):
         self.name = name
@@ -438,6 +471,12 @@ class IniReader:
 
         return shlex.split(new_command.strip())
 
+    def getargv(self, section, name, default=None, replace=True):
+        command = self.getdefault(
+            section, name, default=default, replace=replace)
+
+        return shlex.split(command.strip())
+
     def getbool(self, section, name, default=None):
         s = self.getdefault(section, name, default)
         if s is None:
